package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: API.java 65 2008-07-02 19:47:59Z toni $

/**
 * API versioning.
 * 
 * Versioning is inspired by libtool's rules
 * {@link http://www.gnu.org/software/libtool/manual.html#Versioning}, i.e.
 * <pre>
 * API version = "current"
 * API revision = "revision"
 * API min_supported_revision = "current" - "age"
 * </pre>
 * 
 * Release numbers are meaningful and composed like:
 * <code>Version.MinSupported.Revision.build#</code>,
 * so, e.g. release 2.3.0 won't valid since MinSupported can't be
 * greater than version.
 */
public final class API {
    /**
     * API Version.
     * Newer versions might break backwards compatibility.
     */
    public static final int VERSION = 1;

    /**
     * Revision of the API Version.
     * A newer revision might add features but not change the meaning of
     * existing ones neither remove any of them.
     * @see #VERSION
     */
    public static final int REVISION = 1;

    /**
     * Number of previous Versions supported.
     * For future use.
     * {@see #VERSION}
     * {@see #MIN_SUPPORTED_VERSION}
     */
    public static final int AGE = 0;
    
    /**
     * Minimum API version supported by this implementation.
     * For future use.
     * In principle a new API should provide support for previous revisions,
     * in case it doesn't, this constant will hold the lower supported API.
     * @see #VERSION
     * @pre MIN_SUPPORTED_VERSION &lt;= VERSION
     */
    public static final int MIN_SUPPORTED_VERSION = VERSION - AGE;
}
