package net.outlyer.plugins.utils;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id$

import java.util.Arrays;
import net.outlyer.plugins.Functor;

/**
 * Language extensions provided already by JavaScript.
 * Versions here provide alternative syntax or semantics to the builtin
 * versions, based on other languages.
 * <br />
 * In general {@link LanguageExtensions} will be preferable over this class.
 * @since {@link net.outlyer.plugins.API#REVISION} 1
 * @since {@link #interfaceRevision} 2
 */
public class RedundantLanguageExtensions extends LanguageExtensions {

    public RedundantLanguageExtensions() {
        super(2);
    }

    /**
     * Apply a function to each element of an iterable collection.
     * Note JavaScript 1.6 DOES have a "for each" loop
     * ({@link http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Statements:for_each...in})
     * @see #for_each(Object[], Functor)
     * @param c Collection to which to apply
     * @param f Function to apply
     * @since {@link #interfaceRevision} 2
     */
    public <T> void for_each(final Iterable<T> c, final Functor<Object, T> f) {
        for (final T elem : c) {
            f.apply(elem);
        }
    }
    
    /**
     * Apply a function to each element of an array
     * A JavaScript function can be used transparently as a functor, e.g.:
     * <br />
     * <code>var arr = lang.array(new Array(1, 2, 3));<br />
     * lang.for_each(arr, function(x) { err.println("Element: "+x); });</code>>
     * Will produce:
     * <pre>Element: 1<br />Element: 2<br />Element: 3</pre>
     * @param a Array to which to apply
     * @param f Function to apply
     * @since {@link #interfaceRevision} 2
     */
    public <T> void for_each(final T[] a, final Functor<Object, T> f) {
        if (null == a || null == f) {
            return; // FIXME: Error handling
        }
        for_each(Arrays.asList(a), f); 
    }
}
